@@ -66,7 +66,7 @@ def export_gltf(
     scene,
     include_normals=None,
     merge_buffers=False,
-    unitize_normals=False,
+    unitize_normals=True,
     tree_postprocessor=None,
     embed_buffers=False,
     extension_webp=False,
@@ -85,6 +85,9 @@ def export_gltf(
       Include vertex normals
     merge_buffers : bool
       Merge buffers into one blob.
+    unitize_normals
+      GLTF requires unit normals, however sometimes people
+      want to include non-unit normals for shading reasons.
     resolver : trimesh.resolvers.Resolver
       If passed will use to write each file.
     tree_postprocesser : None or callable
@@ -160,7 +163,7 @@ def export_gltf(
 def export_glb(
     scene,
     include_normals=None,
-    unitize_normals=False,
+    unitize_normals=True,
     tree_postprocessor=None,
     buffer_postprocessor=None,
     extension_webp=False,
@@ -261,6 +264,7 @@ def load_gltf(
     resolver=None,
     ignore_broken=False,
     merge_primitives=False,
+    skip_materials=False,
     **mesh_kwargs,
 ):
     """
@@ -280,6 +284,8 @@ def load_gltf(
     merge_primitives : bool
       If True, each GLTF 'mesh' will correspond
       to a single Trimesh object
+    skip_materials : bool
+      If true, will not load materials (if present).
     **mesh_kwargs : dict
       Passed to mesh constructor
 
@@ -322,13 +328,19 @@ def load_gltf(
         ignore_broken=ignore_broken,
         merge_primitives=merge_primitives,
         mesh_kwargs=mesh_kwargs,
+        skip_materials=skip_materials,
         resolver=resolver,
     )
     return kwargs
 
 
 def load_glb(
-    file_obj, resolver=None, ignore_broken=False, merge_primitives=False, **mesh_kwargs
+    file_obj,
+    resolver=None,
+    ignore_broken=False,
+    merge_primitives=False,
+    skip_materials=False,
+    **mesh_kwargs,
 ):
     """
     Load a GLTF file in the binary GLB format into a trimesh.Scene.
@@ -342,9 +354,15 @@ def load_glb(
       Containing GLB data
     resolver : trimesh.visual.Resolver
       Object which can be used to load other files by name
+    ignore_broken : bool
+      If there is a mesh we can't load and this
+      is True don't raise an exception but return
+      a partial result
     merge_primitives : bool
       If True, each GLTF 'mesh' will correspond to a
       single Trimesh object.
+    skip_materials : bool
+      If true, will not load materials (if present).
 
     Returns
     ------------
@@ -423,6 +441,7 @@ def load_glb(
         buffers=buffers,
         ignore_broken=ignore_broken,
         merge_primitives=merge_primitives,
+        skip_materials=skip_materials,
         mesh_kwargs=mesh_kwargs,
     )
 
@@ -1334,6 +1353,7 @@ def _read_buffers(
     mesh_kwargs,
     ignore_broken=False,
     merge_primitives=False,
+    skip_materials=False,
     resolver=None,
 ):
     """
@@ -1354,6 +1374,8 @@ def _read_buffers(
       a partial result
     merge_primitives : bool
       If true, combine primitives into a single mesh.
+    skip_materials : bool
+      If true, will not load materials (if present).
     resolver : trimesh.resolvers.Resolver
       Resolver to load referenced assets
 
@@ -1433,8 +1455,11 @@ def _read_buffers(
                 # a "sparse" accessor should be initialized as zeros
                 access[index] = np.zeros(count * per_count, dtype=dtype).reshape(shape)
 
-        # load images and textures into material objects
-        materials = _parse_materials(header, views=views, resolver=resolver)
+        # possibly load images and textures into material objects
+        if skip_materials:
+            materials = []
+        else:
+            materials = _parse_materials(header, views=views, resolver=resolver)
 
     mesh_prim = collections.defaultdict(list)
     # load data from accessors into Trimesh objects
@@ -1502,7 +1527,7 @@ def _read_buffers(
                         kwargs["vertex_normals"] = access[attr["NORMAL"]]
                         # do we have UV coordinates
                     visuals = None
-                    if "material" in p:
+                    if "material" in p and not skip_materials:
                         if materials is None:
                             log.debug("no materials! `pip install pillow`")
                         else:
@@ -1558,7 +1583,7 @@ def _read_buffers(
                 mesh_prim[index].append(name)
         except BaseException as E:
             if ignore_broken:
-                log.debug("failed to load mesh", exc_info=True),
+                log.debug("failed to load mesh", exc_info=True)
             else:
                 raise E
 
