@@ -6,6 +6,7 @@ import numpy as np
 from .. import caching, convex, grouping, inertia, transformations, units, util
 from ..exchange import export
 from ..parent import Geometry3D
+from ..typed import Optional
 from ..util import unique_name
 from . import cameras, lighting
 from .transforms import SceneGraph
@@ -252,6 +253,7 @@ class Scene(Geometry3D):
         """
         # avoid accessing attribute in tight loop
         geometry = self.geometry
+        # hash of geometry and transforms
         # start with the last modified time of the scene graph
         hashable = [hex(self.graph.transforms.__hash__())]
         # take the re-hex string of the hash
@@ -954,27 +956,25 @@ class Scene(Geometry3D):
         return png
 
     @property
-    def units(self):
+    def units(self) -> Optional[str]:
         """
-        Get the units for every model in the scene, and
-        raise a ValueError if there are mixed units.
+        Get the units for every model in the scene. If the scene has
+        mixed units or no units this will return None.
 
         Returns
         -----------
-        units : str
-          Units for every model in the scene
+        units
+          Units for every model in the scene or None
+          if there are no units or mixed units
         """
-        existing = [i.units for i in self.geometry.values()]
-
-        if any(existing[0] != e for e in existing):
-            # if all of our geometry doesn't have the same units already
-            # this function will only do some hot nonsense
-            raise ValueError("models in scene have inconsistent units!")
-
-        return existing[0]
+        # get a set of the units of every geometry
+        existing = {i.units for i in self.geometry.values()}
+        if len(existing) == 1:
+            return existing.pop()
+        return None
 
     @units.setter
-    def units(self, value):
+    def units(self, value: str):
         """
         Set the units for every model in the scene without
         converting any units just setting the tag.
@@ -987,7 +987,7 @@ class Scene(Geometry3D):
         for m in self.geometry.values():
             m.units = value
 
-    def convert_units(self, desired, guess=False):
+    def convert_units(self, desired: str, guess: bool = False) -> "Scene":
         """
         If geometry has units defined convert them to new units.
 
